#!/bin/sh

#
# Netborder Express Installation script
# Copyright (c) 2009, Sangoma Technologies Inc.
# ---------------------------------------------
# Jul 20, 2009  Moises Silva  Detect and backup old Netborder installations
# Jul 14, 2009  Moises Silva  removed IP detection, the web interface will take care of that
# May 13, 2009  Moises Silva  added --removeall option, chkconfig support, automatic IP detection and configuration
# Apr 15, 2009  Moises Silva  Remove wanpipe drivers when uninstalling and keep configuration files
# Feb 25, 2009  Moises Silva  Initial version
#

### NOTICE ##
# All environment variables starting with NBE_ (or having NBE_ in its name)
# WILL be exported during installation to be used during the uninstall procedure
# if there is a variable that should not be stored don't use that special keyword
# in its name (ie, the OPT_REMOVE_ALL option)
#############

# Pruduct name (displayed in messages)
NBE_PRODUCT_NAME="Sangoma Netborder Gateway"

# Top root directory
NBE_ROOT_DIR=`pwd`

# Action to execute (received from the command line ./Setup install, remove or whatever)
NBE_ACTION=$1

# Path where the folder netborder/ will be put, --prefix can be specified in the command line to set a different path
NBE_MAIN_PREFIX=/usr/local

# System configuration prefix (where init.d script and sysconfig configuration will be installed)
NBE_SYS_PREFIX=/etc

# Users can decide to NOT install wanpipe, however unless they installed the same version
# that neborder requires, netborder will fail to start (--nowanpipe)
NBE_INSTALL_WANPIPE="YES"

# Debug script --debug
NBE_DEBUG="NO"

# Whether or not to remove everything when uninstalling (even configuration files, licenses etc) DANGEROUS OPTION --removeall!!
OPT_REMOVE_ALL="NO"

# Service names (chkconfig)
NBE_SERVICE_NAME=netborder-gateway
NBE_CONFIG_SERVICE_NAME=netborder-webconfig

# Variable used by ask_user_binary() to return a result
NBE_LAST_USER_ANSWER="INVALID"

function printdbg()
{
  if [ "$NBE_DEBUG" = "YES" ]
  then
    echo $1;
  fi
}

function printmsg()
{
  echo $1
}

function printerr()
{
  echo $1
}

# This function sets the NBE_LAST_USER_ANSWER global variable
# Only possible values are YES or NO
function ask_user_binary()
{
  echo "$1 [yes/no]"
  read answer
  case $answer in
  yes|YES)
    NBE_LAST_USER_ANSWER="YES"
    ;;
  no|NO)
    NBE_LAST_USER_ANSWER="NO"
    ;;
  *)
    printmsg "Please answer yes or no only."
    ask_user_binary "$1"
    ;;
  esac
}


function install_sanity_check()
{

  # check the script is run from the correct directory
  if [ ! -d "$NBE_ROOT_DIR/sangoma" ] || [ ! -f "$NBE_ROOT_DIR/netborder-gateway" ]
  then
    printerr "Current directory $NBE_ROOT_DIR does not look like the installer directory! (sangoma and/or nbe start script missing)"
    exit 1
  fi

  # check the user is root
  if [ "$NBE_INSTALL_WANPIPE" = "YES" ] && [ "$(id -u)" != "0" ]
  then
    printerr "This script must be run as root or run with --nowanpipe!"
    exit 1
  fi

  # check previous installations
  if [ "$NBE_ACTION" = "install" ] 
  then
    service $NBE_SERVICE_NAME status &> /dev/null
    if [ $? = 0 ]
    then
      printerr "$NBE_SERVICE_NAME service is running, please stop it before proceeding. (service $NBE_SERVICE_NAME stop)"
      # TODO: may be we can ask the user if she wants to stop it now and execute service $NBE_SERVICE_NAME stop ourselves or kill -9 $service_pid
      exit 1
    fi
    service $NBE_CONFIG_SERVICE_NAME status &> /dev/null
    if [ $? = 0 ]
    then
      printerr "$NBE_CONFIG_SERVICE_NAME service is running, please stop it before proceeding. (service $NBE_CONFIG_SERVICE_NAME stop)"
      # TODO: may be we can ask the user if she wants to stop it now and execute service $NBE_SERVICE_NAME stop ourselves or kill -9 $service_pid
      exit 1
    fi
  fi

}

function usage()
{
  cat <<ENDOFTEXT 
  USAGE:
  $0 {action}
  or
  $0 {action} {options}

  actions:
  install
  remove

  INSTALL OPTIONS:
  --prefix=<prefix path>
  --nowanpipe Do not build and install Sangoma Wanpipe Driver

  REMOVE OPTIONS:
  --removeall Remove everything related to netborder, including configurations, logs, license files etc. Be careful!!


ENDOFTEXT
  exit 1
}

function valid_action_no_usage()
{
# check for a valid action
# for each of these actions a function execute_action_<action-name> should exist
  printdbg "validating action $1"
  case $1 in
  install)
    printdbg "valid action $1"
    return 0 
    ;;
  remove)
    printdbg "valid action $1"
    return 0;
    ;;
  *)
    printdbg "invalid action $1"
    return 1;
    ;;
  esac
  return 1;
}

function valid_action()
{
  valid_action_no_usage $1 || usage
}

function install_netborder()
{
  printmsg "Installing Netborder Express in $NBE_MAIN_PREFIX/netborder ..."

  if [ -d "$NBE_MAIN_PREFIX/netborder" ]
  then
    # may be we should not backup binaries?
    currdate=$(date +%F-%H-%M-%S)
    backupdir="$NBE_MAIN_PREFIX/netborder-replaced-$currdate"
    mv $NBE_MAIN_PREFIX/netborder $backupdir
    if [ $? != 0 ]
    then
      printerr "Cannot proceed with Netborder installation, could not backup current installation from $NBE_MAIN_PREFIX to $backupdir"
      exit 0
    fi
    # No much point in backing up init.d and sysconfig
    printmsg "Previous Netborder installation found at $NBE_MAIN_PREFIX/netborder, files will be backed up at $backupdir"
  fi

  mkdir -p $NBE_MAIN_PREFIX/netborder
  mkdir -p $NBE_SYS_PREFIX/init.d
  mkdir -p $NBE_SYS_PREFIX/sysconfig

  # set install path (use commas as separator in sed to avoid the need to escape slashed in NBE_MAIN_PREFIX path, yay!)
  eval "sed -i 's,XXX_INSTDIR_XXX,${NBE_MAIN_PREFIX}/netborder,g' $NBE_ROOT_DIR/bin/shttpd.conf"
  eval "sed -i 's,XXX_INSTDIR_XXX,${NBE_MAIN_PREFIX}/netborder,g' $NBE_ROOT_DIR/netborder.conf"

  # fix install.properties
  license_line=$(cat ${NBE_ROOT_DIR}/config/install.properties | grep LicenseFile)
  license_name=`basename $license_line`
  eval "sed -i 's,LicenseFile=\(.*\),LicenseFile=\${NETBORDER_INSTALLDIR}/config/$license_name,g' $NBE_ROOT_DIR/config/linux.install.properties"
  cp $NBE_ROOT_DIR/config/linux.install.properties $NBE_ROOT_DIR/config/install.properties

  NBE_DIRS="Python-2.5/ logs/ lib/ bin/ data/ tone-db/ tones/ oamWebServer/ config/"
  for nbedir in $NBE_DIRS
  do
    cp -r $NBE_ROOT_DIR/$nbedir $NBE_MAIN_PREFIX/netborder
  done
  cp $NBE_ROOT_DIR/netborder.conf $NBE_SYS_PREFIX/sysconfig/
  cp $NBE_ROOT_DIR/netborder-gateway $NBE_SYS_PREFIX/init.d/
  cp $NBE_ROOT_DIR/netborder-webconfig $NBE_SYS_PREFIX/init.d/

  chkconfig --add $NBE_CONFIG_SERVICE_NAME
  # Don't add NBE_SERVICE_NAME since that is up to the user to set

  printmsg "Done!"
  return 0
}

function remove_netborder()
{
  printmsg "Removing Netborder Express from $NBE_MAIN_PREFIX"
  chkconfig --del $NBE_SERVICE_NAME
  chkconfig --del $NBE_CONFIG_SERVICE_NAME
  rm -rf $NBE_MAIN_PREFIX/netborder/bin/
  rm -rf $NBE_MAIN_PREFIX/netborder/lib/
  rm -rf $NBE_MAIN_PREFIX/netborder/Python-2.5/
  rm -rf $NBE_MAIN_PREFIX/netborder/tones/
  rm -rf $NBE_MAIN_PREFIX/netborder/tone-db/
  rm -rf $NBE_MAIN_PREFIX/netborder/tone-package/
  rm -rf $NBE_MAIN_PREFIX/netborder/pstn-config/
  if [ "$OPT_REMOVE_ALL" == "YES" ]
    then
      rm -rf $NBE_SYS_PREFIX/init.d/netborder-gateway
      rm -rf $NBE_SYS_PREFIX/sysconfig/netborder.conf
      # this makes look all previous steps as silly, but oh well :)
      rm -rf $NBE_MAIN_PREFIX/netborder
    fi
}

function remove_wanpipe()
{
  if [ "$NBE_INSTALL_WANPIPE" = "YES" ]
  then
    printmsg "Removing Wanpipe Drivers ..."
    cd sangoma/wanpipe/
    ./Setup remove --silent &> $NBE_ROOT_DIR/wanpipe.log
    if [ $? != 0 ]
    then
        printerr "Wanpipe removal failed!, check wanpipe.log"
        return 1
    fi
    cd -
    printmsg "Done!"
  fi
  return 0
}

function install_wanpipe()
{
  if [ "$NBE_INSTALL_WANPIPE" = "YES" ]
  then
    printmsg "Installing Wanpipe Drivers ..."
    cd sangoma/wanpipe/
    ./Setup install --protocol=AFT_TE1 --silent &> $NBE_ROOT_DIR/wanpipe-install.log
    if [ $? != 0 ]
    then
      printerr "Wanpipe installation failed!, check wanpipe.log"
      return 1
    fi
    printmsg "Done!"
    cd -
  fi
  return 0
}

function parse_opts()
{
  printdbg "checking args"
  for arg in $@
  do
    case $arg in
    --prefix*)
      NBE_MAIN_PREFIX=`echo $arg | cut -d'=' -f2`
      ;;
    --sysprefix*)
      NBE_SYS_PREFIX=`echo $arg | cut -d'=' -f2`
      ;;
    --nowanpipe*)
      NBE_INSTALL_WANPIPE="NO"
      ;;
    --debug*)
      NBE_DEBUG="YES"
      ;;
    --removeall*)
      OPT_REMOVE_ALL="YES"
      ;;
    *)
      valid_action_no_usage $arg
      if [ $? = 1 ]
      then
        printerr "invalid option $arg" 
        usage
        exit
      fi
      ;;
    esac
  done
  return 0
}

function execute_action_install()
{
  rm -f $NBE_ROOT_DIR/installenv.sh
  install_netborder
  install_wanpipe
  # save the environment to be used when removing 
  nbe_env=$(set | grep NBE_)
  echo $nbe_env > $NBE_ROOT_DIR/installenv.sh
  return 0
}

function execute_action_remove()
{
  ask_user_binary "Are you sure you want to remove $NBE_PRODUCT_NAME from you system?"
  if [ "$NBE_LAST_USER_ANSWER" = "NO" ]
  then
    printmsg "Aborting removal due to user request."
    return 1
  fi
  if [ ! -f "$NBE_ROOT_DIR"/installenv.sh ]
  then
    printerr "It seems you have not run install before: (no installenv.sh file found)"
    exit 1
  fi

  # we should be very careful to NOT test for command line affected variables after THIS point
  # otherwise we get wrong values since here we load the install environment
  # a good improvement would be to import just certain variables (for example, variables starting with EXNBE_)
  # that requires to export to installenv.sh just variables starting with that prefix during installation time
  # for now, is just convenient to have it this way, just be aware of the described limitations
  source $NBE_ROOT_DIR/installenv.sh

  if [ "$OPT_REMOVE_ALL" = "YES" ]
  then
    ask_user_binary "--removeall means you will remove even logs, licenses, scripts and EVERYTHING in the $NBE_MAIN_PREFIX/netborder directory. Continue?"
    if [ "$NBE_LAST_USER_ANSWER" = "NO" ]
    then
      printmsg "Aborting removal due to user request."
      return 1
    fi
  fi

  remove_netborder
  remove_wanpipe

  return 0
}

################### main() ########################

# check a valid action is provided
valid_action $NBE_ACTION 

# verify valid options are provided and set the proper variables
parse_opts $@ 

# check we are wherever we are supposed to be and with the proper user etc
install_sanity_check 

# execute the action
execute_action_$NBE_ACTION


